export declare type MixedCtr<T> = Ctr<T> | Record<string, Ctr<T>> | StringConstructor | BooleanConstructor | NumberConstructor;
export declare type StoreOptions = {
    enableWarnings?: boolean;
};
export declare type FieldOptions = {
    nullable?: boolean;
    optional?: boolean;
    isCustom?: boolean;
    producerFields?: {
        [key: string]: string;
    };
};
export declare type FieldProps = {
    key: string;
    type: (() => MixedCtr<any>) | (() => MixedCtr<any>[]);
    options?: FieldOptions;
};
export declare type Ctr<T> = {
    new (...args: any[]): T;
    fields?: FieldProps[];
    of: (data: Partial<T>) => T;
};
export declare function getStore(options?: StoreOptions): {
    set: <T>(type: "unknown" | "mistyped" | "entities", owner: string, key: string) => (updater: (prev: T) => T, initial?: T | undefined) => void;
    init: () => void;
    reset: () => void;
    store: any;
    destroy: () => void;
    register: (owner: string) => void;
    setEntity: (owner: string) => (updater: (prev: any) => any, initial?: any) => void;
};
export declare function mapObjectToEntity<T extends object>(data: Partial<T>, Entity: Ctr<T>): T & Partial<Partial<T>>;
export declare function trackUnknownProps(owner: string, inputKeys: string[], targetKeys: string[]): void;
export declare function isProducerField({ key, type }: FieldProps, producerName: string): any;
export declare function getProducerFields(fields: FieldProps[], producerName: string): FieldProps[];
export declare function produceEntries(initialData: Record<string, any>, fieldsWithProducer: FieldProps[]): Record<string, any>;
export declare function extractInputTypes(fields: (FieldProps & {
    value: any;
})[]): Record<string, string>;
export declare function extractTargetTypes(fields: (FieldProps & {
    value: any;
})[]): Record<string, string>;
export declare function getInputTypes(fields: (FieldProps & {
    value: any;
})[], keysWithProducer: string[]): {
    allInputTypes: Record<string, string>;
    primitiveInputTypes: Record<string, any>;
};
export declare function getTargetTypes(fields: (FieldProps & {
    value: any;
})[], keysWithProducer: string[]): {
    allTargetTypes: Record<string, string>;
    primitiveTargetTypes: Record<string, any>;
};
export declare function trackWrongValues(owner: string, fields: (FieldProps & {
    value: any;
})[], keysWithProducer: string[]): void;
export declare function trackEntity(name: string, fields: (FieldProps & {
    value: any;
})[]): void;
export declare function createProducer<T extends Ctr<T>>(target: T, producerName: string, callBack?: any): (data?: Record<string, any>) => any;
export declare function createEntityStore(options?: StoreOptions): void;
