"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.entity = exports.type = exports.Entity = exports.Producer = exports.Of = void 0;
const core_1 = require("./core");
const typer_1 = require("./typer");
const parser_1 = require("./parser");
const logger_1 = require("./logger");
function Of(type, options) {
    return function (target, key) {
        const owner = target.constructor;
        if (!owner.fields) {
            owner.fields = [];
        }
        owner.fields.push({ key, type, options });
    };
}
exports.Of = Of;
function Producer(target, key) {
    const initial = target[key];
    target[key] = (0, core_1.createProducer)(target, key, initial);
}
exports.Producer = Producer;
function Entity(constructor) {
    Object.assign(constructor, { of: (0, core_1.createProducer)(constructor, "of") });
    return constructor;
}
exports.Entity = Entity;
Entity.of = function () {
    return function (data) {
        return data;
    };
};
function type(type) {
    return function (target, key) {
        const owner = target.constructor;
        if (!owner.types) {
            owner.types = {};
        }
        const _a = type(typer_1.t), { optional: _, nullable: __ } = _a, typeProps = __rest(_a, ["optional", "nullable"]);
        owner.types[key] = typeProps;
    };
}
exports.type = type;
function entity() {
    return function (constr) {
        function of(values) {
            for (const key in values) {
                if (!constr.types[key]) {
                    logger_1.Log.unknown({
                        entityName: constr.name,
                        property: key,
                        received: values[key],
                    });
                    delete values[key];
                    continue;
                }
                const type = constr.types[key];
                let next = (0, parser_1.parse)(type, values[key], { key, owner: constr.name });
                if (!next && (!type.isOptional || !type.isNullable)) {
                    delete values[key];
                    continue;
                }
                else {
                    values[key] = next;
                }
            }
            return Object.assign(new constr(), values);
        }
        Object.assign(constr, { of });
        return constr;
    };
}
exports.entity = entity;
entity.of = function () {
    return function (data) {
        return data;
    };
};
