"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Log = void 0;
const store_1 = require("./store");
const utils_1 = require("./utils");
class Log {
    constructor(message, options) {
        var _a, _b;
        this.message = message;
        this.expected = (_b = (_a = options.expected) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : "";
        this.received = (() => {
            if (typeof options.received === "object") {
                if (options.received === null)
                    return "null";
                if (Object.hasOwn(options.received, "overwrite")) {
                    return options.received.overwrite;
                }
                if (Array.isArray(options.received))
                    return "Array";
                return "Record";
            }
            return (0, utils_1.capitalize)(typeof options.received);
        })();
    }
    static mismatch(options) {
        clearTimeout(Log.timer);
        const warn = new Log("Type Mismatch!", {
            expected: options.expected,
            received: options.received,
        });
        const { entityName, property } = options;
        const { message, expected, received } = warn;
        const store = store_1.Store.getInstance();
        store.set(entityName, property, received);
        if (store.get(entityName, property, received) > 1)
            return;
        setTimeout(() => {
            const count = store.get(entityName, property, received);
            if (typeof window === "undefined") {
                console.log(`${count} x ${message} :: Entity "\x1b[93m${entityName}\x1b[0m" :: Property "\x1b[93m${property}\x1b[0m" :: Expected \x1b[92m${expected}\x1b[0m :: Received \x1b[91m${received}\x1b[0m`);
            }
            else {
                console.log(`${count} x ${message} :: Entity "%c${entityName}%c" :: Property "%c${property}%c" :: Expected %c${expected}%c :: Received %c${received}`, `color: yellow;`, `color: initial;`, `color: yellow;`, `color: initial;`, `color: green;`, `color: initial;`, `color: red;`);
            }
        }, 500);
    }
    static unknown(options) {
        const warn = new Log("Unknown Property!", {
            received: options.received,
        });
        const { entityName, property } = options;
        const { message, received } = warn;
        const store = store_1.Store.getInstance();
        store.set(entityName, property, received);
        if (store.get(entityName, property, received) > 1)
            return;
        setTimeout(() => {
            const count = store.get(entityName, property, received);
            if (typeof window === "undefined") {
                console.log(`${count} x ${message} :: Entity "\x1b[93m${entityName}\x1b[0m" :: Property \x1b[91m${property}: ${received}\x1b[0m`);
            }
            else {
                console.log(`${count} x ${message} :: Entity "%c${entityName}%c" :: Property %c${property}`, `color: yellow;`, `color: initial;`, `color: yellow;`, `color: initial;`, `color: green;`, `color: initial;`, `color: red;`);
            }
        }, 500);
    }
}
exports.Log = Log;
